@@ -8,7 +8,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: session.c,v 1.12 2000/05/03 18:03:07 markus Exp $");
+RCSID("$OpenBSD: session.c,v 1.17 2000/06/05 19:53:40 markus Exp $");
 
 #include "xmalloc.h"
 #include "ssh.h"
@@ -248,7 +248,10 @@ do_authenticated(struct passwd * pw)
 				packet_send_debug("X11 forwarding disabled in server configuration file.");
 				break;
 			}
-#ifdef XAUTH_PATH
+			if (!options.xauth_location) {
+				packet_send_debug("No xauth program; cannot forward with spoofing.");
+				break;
+			}
 			if (no_x11_forwarding_flag) {
 				packet_send_debug("X11 forwarding not permitted for this authentication.");
 				break;
@@ -289,10 +292,6 @@ do_authenticated(struct passwd * pw)
 			fatal_add_cleanup(xauthfile_cleanup_proc, NULL);
 			success = 1;
 			break;
-#else /* XAUTH_PATH */
-			packet_send_debug("No xauth program; cannot forward with spoofing.");
-			break;
-#endif /* XAUTH_PATH */
 
 		case SSH_CMSG_AGENT_REQUEST_FORWARDING:
 			if (no_agent_forwarding_flag || compat13) {
@@ -300,8 +299,7 @@ do_authenticated(struct passwd * pw)
 				break;
 			}
 			debug("Received authentication agent forwarding request.");
-			auth_input_request_forwarding(pw);
-			success = 1;
+			success = auth_input_request_forwarding(pw);
 			break;
 
 		case SSH_CMSG_PORT_FORWARD_REQUEST:
@@ -613,7 +611,8 @@ do_exec_pty(Session *s, const char *command, struct passwd * pw)
 			}
 		}
 		/* Do common processing for the child, such as execing the command. */
-		do_child(command, pw, s->term, s->display, s->auth_proto, s->auth_data, s->tty);
+		do_child(command, pw, s->term, s->display, s->auth_proto,
+		    s->auth_data, s->tty);
 		/* NOTREACHED */
 	}
 	if (pid < 0)
@@ -717,7 +716,10 @@ read_environment_file(char ***env, unsigned int *envsize,
 			fprintf(stderr, "Bad line in %.100s: %.200s\n", filename, buf);
 			continue;
 		}
-		/* Replace the equals sign by nul, and advance value to the value string. */
+		/*
+		 * Replace the equals sign by nul, and advance value to
+		 * the value string.
+		 */
 		*value = '\0';
 		value++;
 		child_set_env(env, envsize, cp, value);
@@ -737,6 +739,7 @@ do_child(const char *command, struct passwd * pw, const char *term,
 {
 	const char *shell, *cp = NULL;
 	char buf[256];
+	char cmd[1024];
 	FILE *f;
 	unsigned int envsize, i;
 	char **env;
@@ -744,6 +747,10 @@ do_child(const char *command, struct passwd * pw, const char *term,
 	struct stat st;
 	char *argv[10];
 
+	/* login(1) is only called if we execute the login shell */
+	if (options.use_login && command != NULL)
+		options.use_login = 0;
+
 	f = fopen("/etc/nologin", "r");
 	if (f) {
 		/* /etc/nologin exists.  Print its contents and exit. */
@@ -862,7 +869,8 @@ do_child(const char *command, struct passwd * pw, const char *term,
 
 	/* read $HOME/.ssh/environment. */
 	if (!options.use_login) {
-		snprintf(buf, sizeof buf, "%.200s/.ssh/environment", pw->pw_dir);
+		snprintf(buf, sizeof buf, "%.200s/.ssh/environment",
+		    pw->pw_dir);
 		read_environment_file(&env, &envsize, buf);
 	}
 	if (debug_flag) {
@@ -944,25 +952,38 @@ do_child(const char *command, struct passwd * pw, const char *term,
 				pclose(f);
 			} else
 				fprintf(stderr, "Could not run %s\n", SSH_SYSTEM_RC);
-		}
-#ifdef XAUTH_PATH
-		else {
+		} else if (options.xauth_location != NULL) {
 			/* Add authority data to .Xauthority if appropriate. */
 			if (auth_proto != NULL && auth_data != NULL) {
-				if (debug_flag)
-					fprintf(stderr, "Running %.100s add %.100s %.100s %.100s\n",
-						XAUTH_PATH, display, auth_proto, auth_data);
-
-				f = popen(XAUTH_PATH " -q -", "w");
+				char *screen = strchr(display, ':');
+				if (debug_flag) {
+					fprintf(stderr,
+					    "Running %.100s add %.100s %.100s %.100s\n",
+					    options.xauth_location, display,
+					    auth_proto, auth_data);
+					if (screen != NULL)
+						fprintf(stderr,
+						    "Adding %.*s/unix%s %s %s\n",
+						    screen-display, display,
+						    screen, auth_proto, auth_data);
+				}
+				snprintf(cmd, sizeof cmd, "%s -q -",
+				    options.xauth_location);
+				f = popen(cmd, "w");
 				if (f) {
-					fprintf(f, "add %s %s %s\n", display, auth_proto, auth_data);
+					fprintf(f, "add %s %s %s\n", display,
+					    auth_proto, auth_data);
+					if (screen != NULL) 
+						fprintf(f, "add %.*s/unix%s %s %s\n",
+						    screen-display, display,
+						    screen, auth_proto, auth_data);
 					pclose(f);
-				} else
-					fprintf(stderr, "Could not run %s -q -\n", XAUTH_PATH);
+				} else {
+					fprintf(stderr, "Could not run %s\n",
+					    cmd);
+				}
 			}
 		}
-#endif /* XAUTH_PATH */
-
 		/* Get the last component of the shell name. */
 		cp = strrchr(shell, '/');
 		if (cp)
@@ -988,7 +1009,8 @@ do_child(const char *command, struct passwd * pw, const char *term,
 				struct stat mailstat;
 				mailbox = getenv("MAIL");
 				if (mailbox != NULL) {
-					if (stat(mailbox, &mailstat) != 0 || mailstat.st_size == 0)
+					if (stat(mailbox, &mailstat) != 0 ||
+					    mailstat.st_size == 0)
 						printf("No mail.\n");
 					else if (mailstat.st_mtime < mailstat.st_atime)
 						printf("You have mail.\n");
